import typer
from dotenv import load_dotenv, find_dotenv
from typing import Optional
from nexio.helpers.jobs import StoreOptions, JobsManagerTodoist

load_dotenv(find_dotenv())
app = typer.Typer()


@app.command()
def pull_jobs(store: StoreOptions = StoreOptions.no_store, output: Optional[str] = None):
    manager = JobsManagerTodoist()
    jobs = manager.get_jobs()
    manager.store(store_options=store, items=jobs, output=output)


@app.command()
def close_jobs():
    manager = JobsManagerTodoist()
    jobs = manager.get_jobs()
    manager.close_jobs(jobs=jobs)


if __name__ == '__main__':
    app()



import os
import psycopg2
from dotenv import load_dotenv, find_dotenv
from pydantic import BaseModel, Field

from nexio.exceptions import MissingEnvironmentVariableError


class PostgresCredentials(BaseModel):
    db_user: str = Field(description="PostgreSQL user name", default="")
    db_password: str = Field(description="PostgreSQL user password", default="")
    db_host: str = Field(description="PostgreSQL host", default="")
    db_port: str = Field(description="PostgreSQL port", default="")
    db_name: str = Field(description="PostgreSQL name", default="")

    def _validate_env_vars(self):
        for var_key, var_value in self.model_dump().items():
            if var_key not in os.environ:
                raise MissingEnvironmentVariableError(f"Missing environment variable {var_key}")

    def get_url(self):
        self._validate_env_vars()
        return f"postgres://{self.db_user}:{self.db_password}@{self.db_host}:{self.db_port}/{self.db_name}"


class Postgres:
    def __init__(self):
        load_dotenv(find_dotenv())
        self._credentials = PostgresCredentials(
            db_user=os.getenv("db_user", ""),
            db_password=os.getenv("db_password", ""),
            db_host=os.getenv("db_host", ""),
            db_name=os.getenv("db_name", ""),
            db_port=os.getenv("db_port", ""),
        )
        self.url = self._credentials.get_url()
        self.conn = None

    def connect(self):
        try:
            self.conn = psycopg2.connect(self.url)

        except psycopg2.Error as e:
            raise e


def main():
    postgres = Postgres()
    postgres.connect()
    print(postgres.conn)


if __name__ == '__main__':
    main()



import os
import pandas as pd
from typing import Optional, List
from enum import Enum

from todoist_api_python.models import Project, Task
from todoist_api_python.api import TodoistAPI
from dotenv import load_dotenv, find_dotenv
from rich import print
from rich.progress import track

load_dotenv(find_dotenv())


class StoreOptions(str, Enum):
    no_store = "no_store"
    csv = "csv"
    postgres = "postgres"


class TodoistHelper:
    def __init__(self):
        self.api = TodoistAPI(os.getenv('TODOIST_API_TOKEN'))

    def get_projects(self):
        try:
            projects = self.api.get_projects()
            return projects

        except Exception as error:
            raise f"Error getting projects: {error}"

    def get_tasks(self, project_ids: Optional[List[str]] = None):
        try:
            tasks = []
            if project_ids is None:
                tasks.extend(self.api.get_tasks())

            # If project_ids provided, use them to filter tasks
            else:
                for project_id in project_ids:
                    tasks.extend(self.api.get_tasks(project_id=project_id))

            return tasks

        except Exception as error:
            raise f"Error getting tasks: {error}"

    def close_tasks(self, task_ids: Optional[List[str]] = None):
        try:
            if task_ids is None:
                raise Exception(f"task_ids not provided and the tasks cannot be closed")

            for task_id in track(task_ids):
                is_success = self.api.close_task(task_id)
                if is_success:
                    print(f"task_id {task_id} successfully closed")

        except Exception as error:
            raise error

    @staticmethod
    def store(store_options: StoreOptions, items: List[Project] | List[Task], output: str):
        if store_options == StoreOptions.csv:
            if output is None:
                raise ValueError("Output cannot be None")

            items = [item.__dict__ for item in items]
            df = pd.DataFrame(items)
            df.to_csv(output, index=False)
            print(df.head(5))

        if store_options == StoreOptions.postgres:
            raise NotImplementedError("Storing to postgres is not implemented yet")



import typer
import os
from todoist_api_python.api import TodoistAPI
from dotenv import load_dotenv, find_dotenv
from typing import Optional, List

from nexio.helpers.todoist import TodoistHelper, StoreOptions

load_dotenv(find_dotenv())
api = TodoistAPI(os.getenv('TODOIST_API_TOKEN'))

app = typer.Typer()


@app.command()
def get_projects(store: StoreOptions = StoreOptions.no_store):
    todoist_helper = TodoistHelper()
    projects = todoist_helper.get_projects()
    todoist_helper.store(store_options=store, items=projects, output="projects.csv")


@app.command()
def get_tasks(
        project_ids: Optional[List[str]] = None,
        store: StoreOptions = StoreOptions.no_store,
        output: str = None
   ):
    todoist_helper = TodoistHelper()
    if project_ids is not None:
        tasks = todoist_helper.get_tasks(project_ids=project_ids)
    else:
        tasks = todoist_helper.get_tasks()
    todoist_helper.store(store_options=store, items=tasks, output=output)




import re
import pandas as pd
from pydantic import BaseModel, Field
from enum import Enum
from typing import List, Optional
from nexio.helpers.todoist import TodoistHelper
from nexio.utils.postgres import Postgres


class StoreOptions(str, Enum):
    no_store = "no_store"
    csv = "csv"
    postgres = "postgres"


class TodoistProject(BaseModel):
    name: str = Field(description="Project Name", default="")
    id: str = Field(description="Project ID", default="")


class TodoistJob(BaseModel):
    name: str = Field(description="Job name", default="")
    saved_at: str = Field(description="Job saved date/time", default="")
    content: str = Field(description="Job content", default="")
    description: str = Field(description="Job description", default="")
    due_at: Optional[str] = Field(description="Job due date/time", default="")
    id: int = Field(description="Job ID", default="")
    label: List[str] = Field(description="Job labels", default="")
    order: int = Field(description="Job order", default=0)
    priority: int = Field(description="Job priority", default=0)
    project_id: str = Field(description="Job's project ID", default="")
    project_name: str = Field(description="Job's project name", default="")
    section_id: Optional[int] = Field(description="Job's section ID", default="")
    url: str = Field(description="Job's URL", default="")


class JobsManagerTodoist:
    def __init__(self):
        self._todoist_helper = TodoistHelper()
        self._project_parent = dict(name="214_job_search", id="2342935426")
        self._project_children = self._get_project_children()

    def _get_project_children(self):
        try:
            project_children = {}
            projects = self._todoist_helper.get_projects()
            for project in projects:
                if project.parent_id == self._project_parent['id']:
                    project_children[project.id] = project.name
            return project_children

        except Exception as e:
            raise f"Error fetching projects from Todoist: {e}"

    def get_jobs(self):
        project_ids = [project_id for project_id in self._project_children.keys()]
        tasks = self._todoist_helper.get_tasks(project_ids=project_ids)
        jobs = []
        for task in tasks:
            name, url = self._parse_content(content=task.content)
            jobs.append(TodoistJob(
                id=task.id,
                name=name,
                saved_at=task.created_at,
                content=task.content,
                description=task.description,
                due_at=str(task.due),
                label=task.labels,
                order=task.order,
                priority=task.priority,
                project_id=task.project_id,
                project_name=self._project_children.get(task.project_id),
                section_id=task.section_id,
                url=url,
            ))

        return jobs

    def close_jobs(self, jobs: List[TodoistJob]):
        task_ids = [str(job.id) for job in jobs]
        self._todoist_helper.close_tasks(task_ids=task_ids)

    @staticmethod
    def _parse_content(content):
        # Regular expression to extract the job_name and url
        pattern = r'\[(.*?)\]\((.*?)\)'

        # Use re.search to extract the groups
        match = re.search(pattern, content)

        if match:
            job_name = match.group(1)
            url = match.group(2)
            return job_name, url

        return "", ""

    @staticmethod
    def store(store_options: StoreOptions, items: List[TodoistJob], output: str):
        if store_options == StoreOptions.csv:
            JobsManagerTodoist._store_to_csv(items, output)

        if store_options == StoreOptions.postgres:
            JobsManagerTodoist._store_to_postgres(jobs=items)

    @staticmethod
    def _store_to_csv(items, output):
        if output is None:
            raise ValueError("Output cannot be None")

        items = [item.model_dump() for item in items]
        df = pd.DataFrame(items)
        # cols = ['name', 'saved_at', 'content', 'description', 'due_at', 'id', 'label',
        #         'order', 'priority', 'project_id', 'project_name', 'section_id', 'url'],
        new_order = ['id', 'name', 'order', 'project_name', 'url', 'saved_at', 'content', 'description', 'due_at',
                     'label', 'priority', 'project_id', 'section_id']
        df = df[new_order]
        df.to_csv(output, index=False)
        print(df.head(5))

    @staticmethod
    def _store_to_postgres(jobs: List[TodoistJob]):
        postgres = Postgres()
        postgres.connect()
        cursor = postgres.conn.cursor()

        insert_query = """
           INSERT INTO jobs (
               id, "order", name, url, label, saved_at, project_id, due_at, description, section_id, priority, content, project_name 
           ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
           ON CONFLICT (id) DO NOTHING;  -- Prevents duplicates based on 'id'
           """

        # Iterate over DataFrame rows and insert them
        for job in jobs:
            try:
                cursor.execute(insert_query, (
                    job.id,
                    job.order,
                    job.name,
                    job.url,
                    job.label,
                    job.saved_at,
                    job.project_id,
                    job.due_at,
                    job.description,
                    job.section_id,
                    job.priority,
                    job.content,
                    job.project_name,
                ))

            except Exception as e:
                raise Exception("Error on row {job.content}: {e}")

        postgres.conn.commit()
        cursor.close()
        postgres.conn.close()
